# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is the Bugzilla Bug Tracking System.
#
# The Initial Developer of the Original Code is Netscape Communications
# Corporation. Portions created by Netscape are
# Copyright (C) 1998 Netscape Communications Corporation. All
# Rights Reserved.
#
# Contributor(s): Alexey Gladkov <legion@altlinux.org>

our $INCLUDE_LOCALE_PATH = "";
our $bugzilla_lang = "";

my @SEARCH_PATH = ( "custom", "default" );

sub sort_array {
    return (sort { (values %{$b})[0]  <=> (values %{$a})[0] } @{$_[0]});
}

sub parseAcceptLanguage {
    my ($elem, @output);
    foreach $elem ( split(',', $_[0]) ) {
	my $tmp = {};
	my @lang = split(';',$elem);
	$tmp->{$lang[0]} = @lang > 1 ? ( split('=',$lang[1]) )[1] : 1;
	push(@output, $tmp);
    }
    return sort_array(\@output);
}

sub getLanguage {
    return "" unless $_[0];
    foreach my $hsh ($_[0]) {
	while (my ($lang, $quality) = each %{$hsh} ) {
	    my @arr = split('-',$lang);
	    if (@arr > 1) {
		return $arr[0] if ($::localizations{$arr[0]});
		return $arr[1] if ($::localizations{$arr[1]});
	    }
	    return $lang if ($::localizations{$lang});
	}
    }
    return ( sort( keys %::localizations ) )[0];
}

sub mkCookieArray {
    return if defined %::COOKIE;
    if (defined $ENV{"HTTP_COOKIE"}) {
	# Don't trust anything which came in as a cookie
	use re 'taint';
	foreach my $pair (split(/;/, $ENV{"HTTP_COOKIE"})) {
    	    $pair = trim($pair);
    	    if ($pair =~ /^([^=]*)=(.*)$/) {
        	$::COOKIE{$1} = $2;
    	    } else {
        	$::COOKIE{$pair} = "";
    	    }
	}
    }
}

sub debug_lang {
    open FD, ">>/tmp/xxx";
    print FD "!!!".$_[0]."\n";
    close FD;
}

if (defined %::localizations) {
    mkCookieArray();
    $bugzilla_lang = $::COOKIE{'Bugzilla_language'};
    if (! $bugzilla_lang && $ENV{'HTTP_ACCEPT_LANGUAGE'}) {
	$bugzilla_lang = getLanguage( parseAcceptLanguage($ENV{'HTTP_ACCEPT_LANGUAGE'}) );
    } elsif ( $bugzilla_lang ) {
	$bugzilla_lang = getLanguage( ( {$bugzilla_lang => 1} ) );
    } else {
	$bugzilla_lang = "en";
    }

    # Make list available languages
    our @languageslist = ();
    while ( my ($key, $value) = each %::localizations ) {
	my $tmp = { value => $key, description => $value };
	push (@languageslist, $tmp);
    }

    # Generate correct path for Template Toolkit
    foreach my $dir (@SEARCH_PATH) {
	$INCLUDE_LOCALE_PATH .= "template/$bugzilla_lang/$dir:";
    }
    chop ($INCLUDE_LOCALE_PATH);
    $INCLUDE_LOCALE_PATH .= ":template/en/custom:template/en/default" unless $bugzilla_lang =~ /en/;
} else {
    $bugzilla_lang = "en";
    $INCLUDE_LOCALE_PATH .= "template/en/custom:template/en/default";
}
